package GEmeTool::DB;

use v5.16.3;

use strict;
use warnings;

use DBI;
use DBD::SQLite;
use DBD::SQLite::Constants qw/:all/;

use GEmeTool::DB::Migrations;
use GEmeTool::Config;

my $dbh;

sub connect {
    if ( defined $dbh ) {
        return $dbh;
    }
    my $class    = shift;
    my $app      = shift;
    my $data_dir = GEmeTool::Config->new->data_dir;
    $dbh = DBI->connect(
        "dbi:SQLite:dbname=@{[$data_dir->child('gemetool.sqlite')]}",
        undef, undef,
        {
            RaiseError => 1,
            sqlite_string_mode => DBD_SQLITE_STRING_MODE_UNICODE_STRICT,
        },
    );
    $class->_migrate($dbh);
    return $dbh;
}

sub _migrate {
    my $class = shift;
    my $dbh   = shift;
    local $dbh->{RaiseError} = 0;
    local $dbh->{PrintError} = 0;
    my @migrations = GEmeTool::DB::Migrations::MIGRATIONS();
    if ( $class->get_current_migration($dbh) > @migrations ) {
        warn "Something happened there, wrong migration number.";
    }
    if ( $class->get_current_migration($dbh) >= @migrations ) {
        say STDERR "Migrations already applied.";
        return;
    }
    $class->_apply_migrations( $dbh, \@migrations );
}

sub _apply_migrations {
    my $class      = shift;
    my $dbh        = shift;
    my $migrations = shift;
    for (
        my $i = $class->get_current_migration($dbh) ;
        $i < @$migrations ;
        $i++
      )
    {
        local $dbh->{RaiseError} = 1;
        my $current_migration = $migrations->[$i];
        my $migration_number  = $i + 1;
        $class->_apply_migration( $dbh, $current_migration, $migration_number );
    }
}

sub _apply_migration {
    my $class             = shift;
    my $dbh               = shift;
    my $current_migration = shift;
    my $migration_number  = shift;
    {
        if (ref $current_migration eq 'CODE') {
            $current_migration->($dbh);
            next;
        }
        $dbh->do($current_migration);
    }
    $dbh->do( <<'EOF', undef, 'current_migration', $migration_number );
INSERT OR REPLACE INTO options
VALUES ($1, $2);
EOF
}

sub get_current_migration {
    my $class  = shift;
    my $dbh    = shift;
    my $result = $dbh->selectrow_hashref( <<'EOF', undef, 'current_migration' );
select value from options where name = ?;
EOF
    return int( $result->{value} // 0 );
}
1;
